// Treiber fr TPIC8101 Klopfsensorinterface
//
// Programmierer:		Peter Diener
// 2010



#include <avr/io.h>

#include "TPIC8101.h"
#include "SPI.h"



// TPIC SPI Chipselect Makro
#define TPIC_select		do { PORT_TPIC_CS &= ~(1<<Pin_TPIC_CS); } while (0)
#define TPIC_deselect	do { PORT_TPIC_CS |=  (1<<Pin_TPIC_CS); } while (0)

// TPIC Integrator Steuermakro
#define	TPIC_hold			do { PORT_TPIC_Int_Hold &= ~(1<<Pin_TPIC_Int_Hold); } while (0)
#define TPIC_run			do { PORT_TPIC_Int_Hold |=  (1<<Pin_TPIC_Int_Hold); } while (0)



// Momentanwerte der Einstellungen
unsigned char TPIC_current_channel;
unsigned char TPIC_current_freq;
unsigned char TPIC_current_gain;
unsigned char TPIC_current_integration_time_constant;



// Datenaustausch ber SPI
unsigned char TPIC_transfer(unsigned char data)
{
	unsigned char answer;

	TPIC_select;										// Whle TPIC8101 als Ziel aus
	answer = SPI_transfer(data);
	TPIC_deselect;									// Whle Ziel ab

	return answer;
}



// Analogen Eingangskanal auswhlen.
// Beispiel: TPIC_select_channel(CH1);
unsigned char TPIC_select_channel (TPIC_ch channel)
{
	TPIC_current_channel = channel;
	return (TPIC_transfer (0b11100000 + channel));
}



// Frequenz des Analogbandpasses einstellen.
// Werte knnen von 0 bis 63 eingestellt werden.
void TPIC_set_bandpass_frequency (unsigned char frequency)
{
	if (frequency > 63) frequency = 63;
	TPIC_current_freq = frequency;
	TPIC_transfer (frequency);
}



// Analoge Verstrkung einstellen.
// Werte knnen von 0 bis 63 eingestellt werden.
void TPIC_set_gain(unsigned char gain)
{
	if (gain > 63) gain = 63;
	TPIC_current_gain = gain;
	TPIC_transfer (0b10000000 + gain);
}



// Integrationszeitkonstante einstellen
// Werte knnen von 0 bis 31 eingestellt werden.
void TPIC_set_integration_time_constant(unsigned char time)
{
	if (time > 31) time = 31;
	TPIC_current_integration_time_constant = time;
	TPIC_transfer (0b11000000 + time);
}



// Takt Vorteiler und SDO Funktion auswhlen.
void TPIC_init(void)
{
	// Porteinstellungen
	TPIC_deselect;																	// Chip select high
	DDR_TPIC_CS |= (1<<Pin_TPIC_CS);								// Chip select ist ein Ausgang
	PORT_TPIC_test |= (1<<Pin_TPIC_test);						// Test Pin high
	DDR_TPIC_test  |= (1<<Pin_TPIC_test);						// Test Pin ist ein Ausgang
	TPIC_hold;
	DDR_TPIC_Int_Hold |= (1<<Pin_TPIC_Int_Hold);		// Int_hold ist ein Ausgang

	// Taktfrequenz einstellen
	TPIC_transfer (TPIC_prescaler_SDO);

	// Advanced SPI Mode auswhlen
	TPIC_transfer (0b01110001);

	// Initialisierungswerte einstellen
	TPIC_select_channel(CH1);

  TPIC_set_bandpass_frequency(TPIC_freq_5_29_kHz);

  TPIC_set_gain(TPIC_gain_1_000);		//TBD

  // Niedrigere Zeitkonstante bedeutet schnelleren Anstieg, also grere
  // Empfindlichkeit (greres Ergebnis bei gleichem Signal)
  TPIC_set_integration_time_constant(TPIC_intTime_150_us);		//TBD
}



// Klopflautstrke auslesen und Integrator rcksetzen
unsigned int TPIC_get_knock_value(void)
{
	unsigned int knock_value;

	TPIC_hold;
	TPIC_transfer (TPIC_prescaler_SDO);																	// Auslesen von LSB initiieren
	knock_value  = TPIC_transfer (TPIC_prescaler_SDO);									// LSB auslesen
	TPIC_select_channel (TPIC_current_channel);													// Auslesen von MSB initiieren
	knock_value |= ((TPIC_select_channel (TPIC_current_channel) & 0b11000000) << 2);		// MSB auslesen
	TPIC_run;
	return knock_value;
}
